# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.10
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.





from sys import version_info as _swig_python_version_info
if _swig_python_version_info >= (2, 7, 0):
    def swig_import_helper():
        import importlib
        pkg = __name__.rpartition('.')[0]
        mname = '.'.join((pkg, '_fiftyone_degrees_mobile_detector_v3_trie_wrapper')).lstrip('.')
        try:
            return importlib.import_module(mname)
        except ImportError:
            return importlib.import_module('_fiftyone_degrees_mobile_detector_v3_trie_wrapper')
    _fiftyone_degrees_mobile_detector_v3_trie_wrapper = swig_import_helper()
    del swig_import_helper
elif _swig_python_version_info >= (2, 6, 0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_fiftyone_degrees_mobile_detector_v3_trie_wrapper', [dirname(__file__)])
        except ImportError:
            import _fiftyone_degrees_mobile_detector_v3_trie_wrapper
            return _fiftyone_degrees_mobile_detector_v3_trie_wrapper
        if fp is not None:
            try:
                _mod = imp.load_module('_fiftyone_degrees_mobile_detector_v3_trie_wrapper', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _fiftyone_degrees_mobile_detector_v3_trie_wrapper = swig_import_helper()
    del swig_import_helper
else:
    import _fiftyone_degrees_mobile_detector_v3_trie_wrapper
del _swig_python_version_info
try:
    _swig_property = property
except NameError:
    pass  # Python < 2.2 doesn't have 'property'.

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if (name == "thisown"):
        return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method:
        return method(self, value)
    if (not static):
        if _newclass:
            object.__setattr__(self, name, value)
        else:
            self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)


def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)


def _swig_getattr(self, class_type, name):
    if (name == "thisown"):
        return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method:
        return method(self)
    raise AttributeError("'%s' object has no attribute '%s'" % (class_type.__name__, name))


def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except __builtin__.Exception:
    class _object:
        pass
    _newclass = 0

class SwigPyIterator(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, SwigPyIterator, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, SwigPyIterator, name)

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _fiftyone_degrees_mobile_detector_v3_trie_wrapper.delete_SwigPyIterator
    __del__ = lambda self: None

    def value(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.SwigPyIterator_value(self)

    def incr(self, n=1):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.SwigPyIterator_incr(self, n)

    def decr(self, n=1):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.SwigPyIterator_decr(self, n)

    def distance(self, x):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.SwigPyIterator_distance(self, x)

    def equal(self, x):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.SwigPyIterator_equal(self, x)

    def copy(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.SwigPyIterator_copy(self)

    def next(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.SwigPyIterator_next(self)

    def __next__(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.SwigPyIterator___next__(self)

    def previous(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.SwigPyIterator_previous(self)

    def advance(self, n):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.SwigPyIterator_advance(self, n)

    def __eq__(self, x):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.SwigPyIterator___eq__(self, x)

    def __ne__(self, x):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.SwigPyIterator___isub__(self, n)

    def __add__(self, n):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.SwigPyIterator___add__(self, n)

    def __sub__(self, *args):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self
SwigPyIterator_swigregister = _fiftyone_degrees_mobile_detector_v3_trie_wrapper.SwigPyIterator_swigregister
SwigPyIterator_swigregister(SwigPyIterator)

class MapStringString(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, MapStringString, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, MapStringString, name)
    __repr__ = _swig_repr

    def iterator(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __nonzero__(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString___nonzero__(self)

    def __bool__(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString___bool__(self)

    def __len__(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString___len__(self)
    def __iter__(self):
        return self.key_iterator()
    def iterkeys(self):
        return self.key_iterator()
    def itervalues(self):
        return self.value_iterator()
    def iteritems(self):
        return self.iterator()

    def __getitem__(self, key):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString___getitem__(self, key)

    def __delitem__(self, key):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString___delitem__(self, key)

    def has_key(self, key):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_has_key(self, key)

    def keys(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_keys(self)

    def values(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_values(self)

    def items(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_items(self)

    def __contains__(self, key):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString___contains__(self, key)

    def key_iterator(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_key_iterator(self)

    def value_iterator(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_value_iterator(self)

    def __setitem__(self, *args):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString___setitem__(self, *args)

    def asdict(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_asdict(self)

    def __init__(self, *args):
        this = _fiftyone_degrees_mobile_detector_v3_trie_wrapper.new_MapStringString(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this

    def empty(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_empty(self)

    def size(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_size(self)

    def swap(self, v):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_swap(self, v)

    def begin(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_begin(self)

    def end(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_end(self)

    def rbegin(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_rbegin(self)

    def rend(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_rend(self)

    def clear(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_clear(self)

    def get_allocator(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_get_allocator(self)

    def count(self, x):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_count(self, x)

    def erase(self, *args):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_erase(self, *args)

    def find(self, x):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_find(self, x)

    def lower_bound(self, x):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_lower_bound(self, x)

    def upper_bound(self, x):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_upper_bound(self, x)
    __swig_destroy__ = _fiftyone_degrees_mobile_detector_v3_trie_wrapper.delete_MapStringString
    __del__ = lambda self: None
MapStringString_swigregister = _fiftyone_degrees_mobile_detector_v3_trie_wrapper.MapStringString_swigregister
MapStringString_swigregister(MapStringString)

class VectorString(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, VectorString, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, VectorString, name)
    __repr__ = _swig_repr

    def iterator(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __nonzero__(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString___nonzero__(self)

    def __bool__(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString___bool__(self)

    def __len__(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString___len__(self)

    def __getslice__(self, i, j):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString___getslice__(self, i, j)

    def __setslice__(self, *args):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString___setslice__(self, *args)

    def __delslice__(self, i, j):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString___delslice__(self, i, j)

    def __delitem__(self, *args):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString___delitem__(self, *args)

    def __getitem__(self, *args):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString___getitem__(self, *args)

    def __setitem__(self, *args):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString___setitem__(self, *args)

    def pop(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_pop(self)

    def append(self, x):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_append(self, x)

    def empty(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_empty(self)

    def size(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_size(self)

    def swap(self, v):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_swap(self, v)

    def begin(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_begin(self)

    def end(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_end(self)

    def rbegin(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_rbegin(self)

    def rend(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_rend(self)

    def clear(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_clear(self)

    def get_allocator(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_get_allocator(self)

    def pop_back(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_pop_back(self)

    def erase(self, *args):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_erase(self, *args)

    def __init__(self, *args):
        this = _fiftyone_degrees_mobile_detector_v3_trie_wrapper.new_VectorString(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this

    def push_back(self, x):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_push_back(self, x)

    def front(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_front(self)

    def back(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_back(self)

    def assign(self, n, x):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_assign(self, n, x)

    def resize(self, *args):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_resize(self, *args)

    def insert(self, *args):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_insert(self, *args)

    def reserve(self, n):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_reserve(self, n)

    def capacity(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_capacity(self)
    __swig_destroy__ = _fiftyone_degrees_mobile_detector_v3_trie_wrapper.delete_VectorString
    __del__ = lambda self: None
VectorString_swigregister = _fiftyone_degrees_mobile_detector_v3_trie_wrapper.VectorString_swigregister
VectorString_swigregister(VectorString)

class Match(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Match, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Match, name)

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    __swig_destroy__ = _fiftyone_degrees_mobile_detector_v3_trie_wrapper.delete_Match
    __del__ = lambda self: None

    def getValues(self, *args):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Match_getValues(self, *args)

    def getValue(self, *args):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Match_getValue(self, *args)

    def getDeviceId(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Match_getDeviceId(self)

    def getRank(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Match_getRank(self)

    def getDifference(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Match_getDifference(self)

    def getMethod(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Match_getMethod(self)

    def getUserAgent(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Match_getUserAgent(self)
Match_swigregister = _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Match_swigregister
Match_swigregister(Match)

class Provider(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Provider, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Provider, name)
    __repr__ = _swig_repr
    __swig_destroy__ = _fiftyone_degrees_mobile_detector_v3_trie_wrapper.delete_Provider
    __del__ = lambda self: None

    def getHttpHeaders(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Provider_getHttpHeaders(self)

    def getAvailableProperties(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Provider_getAvailableProperties(self)

    def getDataSetName(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Provider_getDataSetName(self)

    def getDataSetFormat(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Provider_getDataSetFormat(self)

    def getDataSetPublishedDate(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Provider_getDataSetPublishedDate(self)

    def getDataSetNextUpdateDate(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Provider_getDataSetNextUpdateDate(self)

    def getDataSetSignatureCount(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Provider_getDataSetSignatureCount(self)

    def getDataSetDeviceCombinations(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Provider_getDataSetDeviceCombinations(self)

    def getMatch(self, *args):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Provider_getMatch(self, *args)

    def getMatchJson(self, *args):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Provider_getMatchJson(self, *args)

    def reloadFromFile(self):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Provider_reloadFromFile(self)

    def reloadFromMemory(self, source, length):
        return _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Provider_reloadFromMemory(self, source, length)

    def __init__(self, *args):
        this = _fiftyone_degrees_mobile_detector_v3_trie_wrapper.new_Provider(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
Provider_swigregister = _fiftyone_degrees_mobile_detector_v3_trie_wrapper.Provider_swigregister
Provider_swigregister(Provider)

# This file is compatible with both classic and new-style classes.


